import fastify from 'fastify'
import formBodyPlugin from '@fastify/formbody'
import fastifySwagger from '@fastify/swagger'
import fastifyMetrics from 'fastify-metrics'
import helmet from '@fastify/helmet'
import yaml from 'js-yaml'
import fs from 'fs'

import versionHandler from './handlers/version.js'
import healthPlugin from './plugins/health.js'
import manifestsPlugin from './plugins/manifests.js'
import metadataPlugin from './plugins/metadata.js'
import redirectsPlugin from './plugins/redirects.js'
import serveFilePlugin from './plugins/serve-files.js'
import { logger } from './logger.js'

const swaggerDocument = yaml.load(fs.readFileSync('./src/swagger.yaml', 'utf8'))

export async function createApp (basePath) {
  const app = fastify({
    logger,
    connectionTimeout: 30000
  })
  await app.register(formBodyPlugin)
  await app.register(helmet, {
    contentSecurityPolicy: false,
    crossOriginEmbedderPolicy: false,
    originAgentCluster: false,
    crossOriginOpenerPolicy: { policy: 'same-origin-allow-popups' }
  })
  await app.register(healthPlugin)
  await app.register(fastifyMetrics, {
    routeMetrics: {
      routeBlacklist: ['/ready', '/live', '/health', '/healthy', '/metrics', '/api-docs']
    }
  })
  await app.register(fastifySwagger, {
    routePrefix: '/api-docs',
    prefix: process.env.APP_ROOT,
    swagger: swaggerDocument,
    exposeRoute: process.env.EXPOSE_API_DOCS === 'true'
  })

  await app.addHook('preHandler', versionHandler)

  await app.register(manifestsPlugin, { prefix: process.env.APP_ROOT })
  await app.register(metadataPlugin, { prefix: process.env.APP_ROOT })
  await app.register(redirectsPlugin, { prefix: process.env.APP_ROOT })

  await app.register(serveFilePlugin, { prefix: process.env.APP_ROOT })

  return app
}
